************************************************************************
*                                                                      *
*              Sample user written modules for the                     *
*                 Merlin optimization package.                         *
*                                                                      *
************************************************************************
*                                                                      *
*  This file contains the appropriate user written modules for         *
*  the Rosenbrock test function:                                       *
*    Rosenbrock H.H, An automatic method for finding the greatest or   *
*    least value of a function, Computer J., 5 (1962) 147-151.         *
*    More J.J, Garbow B.S. and Hillstrom K.E., Testing unconstrained   *
*    optimization software, ACM Trans. Math. Softw., 7 (1981) 17-41.   *
*  The Rosenbrock function has 2 variables, 2 terms.                   *
*                                                                      *
*  The following modules are included:                                 *
*    PROGRAM MASTER             A sample main program.                 *
*    FUNCTION FUNMIN            The Rosenbrock function coded as a     *
*                               general function.                      *
*    SUBROUTINE SUBSUM          The Rosenbrock function coded as a     *
*                               sum-of-squares function.               *
*    SUBROUTINE GRANAL          First partial derivatives.             *
*    SUBROUTINE HANAL           Second partial derivatives.            *
*    SUBROUTINE JANAL           The Jacobian matrix (of little use     *
*                               in this example since this is not a    *
*                               least squares fit).                    *
*                                                                      *
*  Note: This file must be processed by the Merlin/MCL Installer       *
*        before it can be compiled.                                    *
*                                                                      *
*  More information on the Merlin optimization environment is          *
*  available at: http://nrt.cs.uoi.gr/merlin                           *
*                                                                      *
************************************************************************
      PROGRAM MASTER
      IMPLICIT $(TYPE) (A-H,O-Z)
C
C  Maximum number of variables.
      PARAMETER ( MXV = $(MXV) )
C  Maximum number of terms.
      PARAMETER ( MXT = $(MXT) )
C
C  The 2-dimensional storage required by some Merlin commands depends
C  on the maximum number of parameters (MXV) and the maximum number of
C  terms (MXT).
C  If MXV+11 > MXT one needs MAX1 storage locations:
      PARAMETER ( MAX1 = MXV**2+11*MXV )
C  Otherwise:
      PARAMETER ( MAX2 = MXV*MXT )
C  For this example we chose MAX1 (actually MAX2 would be ok too).
      PARAMETER ( MAXW = MAX1 )
C  Alternatively if you don't have that much storage available,
C  set MAXW to any positive value. Commands that need more storage
C  will not be executed by Merlin.
      DIMENSION VERSIM(MAXW)
C
      WRITE (*,*) 'Enter number of variables, number of squared terms:'
      READ (*,*) N, M
      CALL MERLIN(N,M,VERSIM,MAXW,IQUIT)
C
      END
C  ---------------------------------------------------------------------
      FUNCTION FUNMIN ( X, N )
C  ---------------------------------------------------------------------
      IMPLICIT $(TYPE) (A-H,O-Z)
      DIMENSION X(N)
      X1 = X(1)
      X2 = X(2)
      FUNMIN = (10*(X2-X1**2))**2 + (1-X1)**2
      END
C  ---------------------------------------------------------------------
      SUBROUTINE SUBSUM ( M, N, X, F )
C  ---------------------------------------------------------------------
      IMPLICIT $(TYPE) (A-H,O-Z)
      DIMENSION X(N), F(M)
      X1 = X(1)
      X2 = X(2)
      F(1) = 10*(X2-X1**2)
      F(2) = 1-X1
      END
C  ---------------------------------------------------------------------
      SUBROUTINE GRANAL ( N, X, GRAD )
C  ---------------------------------------------------------------------
      IMPLICIT $(TYPE) (A-H,O-Z)
      DIMENSION X(N), GRAD(N)
      X1 = X(1)
      X2 = X(2)
      T = X2-X1**2
      GRAD(1) = -400*X1*T - 2*(1-X1)
      GRAD(2) = 200*T
      END
C  ---------------------------------------------------------------------
      SUBROUTINE HANAL ( H, LD, N, X )
C  ---------------------------------------------------------------------
C  Note: We only have to fill the lower triangular (including the
C  diagonal) part of matrix H.
C  ---------------------------------------------------------------------
      IMPLICIT $(TYPE) (A-H,O-Z)
      DIMENSION H(LD,N), X(N)
      X1 = X(1)
      X2 = X(2)
      H(1,1) = 1200*X1**2 - 400*X2 + 2
      H(2,1) = -400*X1
      H(2,2) = 200
      END
C  ---------------------------------------------------------------------
      SUBROUTINE JANAL ( M, N, X, FJ, LD )
C  ---------------------------------------------------------------------
      IMPLICIT $(TYPE) (A-H,O-Z)
      DIMENSION X(N), FJ(LD,N)
      X1 = X(1)
      X2 = X(2)
      FJ(1,1) = -20*X1
      FJ(1,2) = 10
      FJ(2,1) = -1
      FJ(2,2) = 0
      END
